package mschmidt.komo.datatools;

/*
 * Instances of this wizard object class control the database backup process.
 */

import java.io.File;
import java.util.ArrayList;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.wizard.Wizard;

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationImages;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.preferences.PreferenceConstants;

public class BackupWizard extends Wizard {

	// Instantiate the data model
	BackupData data = new BackupData();

	private IPreferenceStore store = KomoPlugin.getDefault()
			.getPreferenceStore();

	/**
	 * Constructor to create the wizard and initializes variables.
	 */
	public BackupWizard() {

		String engineString = new String();
		setWindowTitle("Database Maintenance");
		setDefaultPageImageDescriptor(KomoPlugin
				.getImageDescriptor(ApplicationImages.DB_BACKUP32_ICON));

		// Create dialog settings if they don't exist, then set them.
		IDialogSettings dlgSettings = KomoPlugin.getDefault()
				.getDialogSettings().getSection("BackupSection");
		if (dlgSettings == null) {
			dlgSettings = KomoPlugin.getDefault().getDialogSettings()
					.addNewSection("BackupSection");
			dlgSettings.put(WizardConstants.BACKUP_DISABLE_DOLLAR, false);
			dlgSettings.put(WizardConstants.BACKUP_FILE, "");
			dlgSettings.put(WizardConstants.BACKUP_FORMAT, "Custom");
			dlgSettings.put(WizardConstants.BACKUP_IGNORE_VERSION, false);
			dlgSettings.put(WizardConstants.BACKUP_VERBOSE, true);
			dlgSettings.put(WizardConstants.BACKUP_WITH_OIDS, false);

			// Attempt to find the pg_dump file.
			engineString = getEngine();
			dlgSettings.put(WizardConstants.BACKUP_ENGINE, engineString);
		}

		setDialogSettings(dlgSettings);

		/*
		 * If this wizard is initially opened before the JDBC is entered, the
		 * backup engine may not be in the Wizard Constants. Try to find it if
		 * it isn't present.
		 */
		engineString = getDialogSettings().get(WizardConstants.BACKUP_ENGINE);
		if (engineString == null || engineString.length() == 0) {
			engineString = getEngine();
			if (engineString != null && engineString.length() > 0) {
				getDialogSettings().put(WizardConstants.BACKUP_ENGINE,
						engineString);
			}
		}

		data.backupEngine = getDialogSettings().get(
				WizardConstants.BACKUP_ENGINE);
		data.backupFile = getDialogSettings().get(WizardConstants.BACKUP_FILE);
		data.backupFormat = getDialogSettings().get(
				WizardConstants.BACKUP_FORMAT);
		data.disableDollar = getDialogSettings().getBoolean(
				WizardConstants.BACKUP_DISABLE_DOLLAR);
		data.doVerbose = getDialogSettings().getBoolean(
				WizardConstants.BACKUP_VERBOSE);
		data.doWithOids = getDialogSettings().getBoolean(
				WizardConstants.BACKUP_WITH_OIDS);
		data.ignoreVersion = getDialogSettings().getBoolean(
				WizardConstants.BACKUP_IGNORE_VERSION);
	}

	/*
	 * Add page to the wizard.
	 */
	@Override
	public final void addPages() {
		BackupWizardPage page = new BackupWizardPage();
		addPage(page);
	}

	/*
	 * Try to get the backup engine. @return the backup engine or an empty
	 * string if it can't be found.
	 */
	private String getEngine() {
		String retString = new String();
		String tempString = store.getString(PreferenceConstants.JDBC_PATH);
		if (tempString != null && tempString.length() > 0) {
			File tempFile = new File(tempString);
			if (tempFile.isFile()) {
				tempString = tempFile.getParent();
				tempFile = new File(tempString);
				if (tempFile.isDirectory()) {
					tempString = tempFile.getParent();
					tempFile = new File(tempString);
					if (tempFile.isDirectory()) {
						tempString = tempString + File.separator + "bin"
								+ File.separator + "pg_dump.exe";
						tempFile = new File(tempString);
						if (tempFile.isFile()) {
							retString = tempString;
						}
					}
				}
			}
		}
		return retString;
	}

	/*
	 * (non-Javadoc) Processing that occurs when the Finish button is pressed.
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	@Override
	public final boolean performFinish() {
		boolean allOK = true;

		// Save dialog settings
		if (getDialogSettings() != null) {
			getDialogSettings().put(WizardConstants.BACKUP_DISABLE_DOLLAR,
					data.disableDollar);
			getDialogSettings().put(WizardConstants.BACKUP_ENGINE,
					data.backupEngine);
			getDialogSettings().put(WizardConstants.BACKUP_FILE,
					data.backupFile);
			getDialogSettings().put(WizardConstants.BACKUP_FORMAT,
					data.backupFormat);
			getDialogSettings().put(WizardConstants.BACKUP_IGNORE_VERSION,
					data.ignoreVersion);
			getDialogSettings().put(WizardConstants.BACKUP_VERBOSE,
					data.doVerbose);
			getDialogSettings().put(WizardConstants.BACKUP_WITH_OIDS,
					data.doWithOids);
		}

		// Warn user if backup will overwrite an existing file
		File file = new File(data.backupFile);
		if (file.isFile()) {
			allOK = MessageDialog
					.openQuestion(
							getShell(),
							"Backup Warning",
							"The selected backup file already exists\nand will be replaced.\nDo you want to continue?");
		}

		// Check for password and then run backup job
		if (allOK) {
			if (Application.dbPassword.checkPassword()) {

				// Initialize an array list to collect the parameters.
				ArrayList<String> parmList = new ArrayList<String>();

				parmList.add(data.backupEngine);

				parmList.add("--host="
						+ store.getString(PreferenceConstants.DB_URL));

				parmList.add("--port="
						+ store.getString(PreferenceConstants.DB_PORT));

				parmList.add("--user="
						+ store.getString(PreferenceConstants.DB_USER));

				parmList.add("-W");

				if (data.ignoreVersion) {
					parmList.add("-i");
				}
				if (data.doWithOids) {
					parmList.add("-o");
				}
				if (data.disableDollar) {
					parmList.add("--disable-dollar-quoting");
				}
				if (data.doVerbose) {
					parmList.add("-v");
				}
				if (data.backupFormat.equals("Tar")) {
					parmList.add("-Ft");
				} else {
					parmList.add("-Fc");
				}

				parmList.add("--file=" + data.backupFile);

				parmList.add(store.getString(PreferenceConstants.DB_NAME));

				// place the list in an array to pass to the job.
				int listSize = parmList.size();
				String[] parmArray = new String[listSize];
				for (int k = 0; k < listSize; ++k) {
					parmArray[k] = parmList.get(k);
				}

				// Create and schedule the job.
				Job backupJob = new UtilityProcessJob("Backup",
						ApplicationImages.DB_BACKUP_ICON, parmArray, null, 1,
						false);
				backupJob.schedule();
			}
		}
		return allOK;
	}

}
