package mschmidt.komo.datatools;

/*
 * Instances of this object class are used to perform
 * runtime-based utility operations (backup and restore).
 * These tasks are performed in a helper thread.
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationActionBarAdvisor;
import mschmidt.komo.DbInfoDialog;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.preferences.PreferenceConstants;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.progress.IProgressConstants;

public class UtilityProcessJob extends Job {

	// Initialize variables used by jobs.
	private String dropApp = new String();

	private String pwString = Application.dbPassword.getPassword();
	
	private String thisImage = new String();

	private String thisJob;
	
	private int pwPrompts;

	// set an arbitrary value for monitor.worked()
	private int monitorWorked = 10;

	private String[] cmdParms;
	
	private StringBuffer sbInfo = new StringBuffer();

	private BufferedReader procReader = null;

	private PrintWriter procWriter = null;

	boolean doAnalyze;
	
	boolean runOK = false;
	
	public UtilityProcessJob(final String jobName, final String imageString,
			final String[] parmList, String appString, int pwNum, boolean requestAnalyze) {
		super(jobName);
		thisJob = jobName;
		thisImage = imageString;
		cmdParms = parmList;
		dropApp = appString;
		pwPrompts = pwNum;
		doAnalyze = requestAnalyze;
	}

	@Override
	protected final IStatus run(final IProgressMonitor monitor) {
		if (thisImage != null && thisImage.length() > 0) {
			setProperty(IProgressConstants.ICON_PROPERTY, KomoPlugin
					.getImageDescriptor(thisImage));
		}

		try {
			sbInfo.append("*** Execute " + thisJob + " Operation:\n");
			// Make sure password and application exist.
			if (pwString == null || pwString.length() == 0) {
				sbInfo.append(
					"Could not obtain password.\n\n*** Operation canceled.\n");
				return Status.CANCEL_STATUS;
			}
			File tempFile = new File(cmdParms[0]);
			if (!tempFile.isFile()) {
				sbInfo.append(
					"Could not access backup engine file.\n\n*** Operation canceled.\n");
				return Status.CANCEL_STATUS;
			}
			sbInfo.append("\n*** Begin Processing Log\n");
			
			monitor.beginTask("Database " + thisJob, IProgressMonitor.UNKNOWN);
			/*
			 * Although processes do not connect to the database through JDBC,
			 * to be entirely safe a database lock is acquired.
			 */
			Application.dbConn.dbLock.acquire();
			
			/*
			 * For a full restore, it is necessary to delete
			 * the current database. 
			 */
			if (dropApp != null && dropApp.length() > 0) {
				if (!dbDropper(monitor)) {
					return Status.CANCEL_STATUS;
				}
			}
			if (!mainProcess(monitor)) {
				return Status.CANCEL_STATUS;
			}

			// Perform Analyze operation, if requested.
			if (doAnalyze) {
				if (!dbAnalyze(monitor)) {
					return Status.CANCEL_STATUS;
				}
			}
			runOK = true;
		} catch (Exception e) {
			sbInfo.append(e.getMessage() + "\n");
			e.printStackTrace();

		} finally {
			Application.dbConn.dbLock.release();
			sbInfo.append("\n*** End Processing Log.\n");
			monitor.done();
			showResults(sbInfo.toString());
		}
		return Status.OK_STATUS;
	}
	
	/*
	 * Perform the main process.
	 */
	private boolean mainProcess(final IProgressMonitor monitor) {
		// Instantiate variable needed to run the process.
		Process mainProc = null;
		// Initialize a boolean to track the method execution.
		boolean mainOK = false;
		
		try {
			// Show the restore operation command line in the output log.
			String line = new String();
			int tempInt = cmdParms.length;
			for (int k = 0; k < tempInt; ++k) {
				line = line + cmdParms[k] + " ";
			}
			sbInfo.append("...Begin database restore operation.\nCommand is: " 
				+ line + "\n");
			
			/*
			 * Use ProcessBuilder class to set up the main process.
			 * @since JDK 5.0.
			 */
			ProcessBuilder pb = new ProcessBuilder(cmdParms);
			mainProc = pb.start();

			/*
			 * Put a buffered reader on stderr - 
			 * this is where the PostgreSQL log is output. 
			 */
			procReader = new BufferedReader(new InputStreamReader(mainProc
				.getErrorStream())); 
		
			// Set up a writer to output the password.
			procWriter = new PrintWriter(mainProc.getOutputStream());

			/*
			 * Send password.  Cancel operation if a problem occurs.
			 */
			if (!sendPassword(monitor)) {
				return mainOK;
			}
		
			/*
			 * The Restore with Create process prompts twice for the password.
			 * After the first password is entered, it outputs a cr/lf then is 
			 * inactive.  The readLine() deals with this behavior and prevents
			 * the process from hanging.
			 */
			if (pwPrompts == 2) {
				procReader.readLine();
				if (!sendPassword(monitor)) {
					return mainOK;
				}
			}
		
			/*
			 * Collect output log from the process.
			 */
			line = "";
			while ((line = procReader.readLine()) != null) {
				sbInfo.append(line + "\n");
				monitor.worked(monitorWorked);
			}
		
			// Make sure the process is finished.
			if (mainProc.waitFor() == 0) {
				mainOK = true;
			}
		} catch (Exception e) {
			sbInfo.append(e.getMessage() + "\n");
			e.printStackTrace();
			
		} finally {
			// Clean up and return to the main run method.
			if (procReader != null) {
				try {
					procReader.close();
				} catch (IOException e) {
					sbInfo.append(e.getMessage());
					e.printStackTrace();
				}
			}
			if (procWriter != null) {
				procWriter.close();
			}
			if (mainProc != null) {
				mainProc.destroy();
			}
			sbInfo.append("...End database restore operation.\n\n");
			if (!mainOK) {
				sbInfo.append("\n*** Database restore operation failed.\n");
			}
		}
		return mainOK;
	}
	
	/*
	 * Perform the database analyze operation.
	 */
	private boolean dbAnalyze(final IProgressMonitor monitor) {
		// The variables needed to perform this method.
		Statement sqlStmt = null;
		Connection thisConnection = Application.dbConn.getConnection();

		// Initialize a boolean to track the method execution. 
		boolean analyzeOK = false;
		
		try {
			sbInfo.append("...Begin analyze operation.\nCommand is: ANALYZE VERBOSE\n");
			if (thisConnection == null) {
				sbInfo.append("Could not establish database connection.\n");
				return analyzeOK;
			}
			sqlStmt = thisConnection.createStatement();
			sqlStmt.execute("ANALYZE VERBOSE");
			SQLWarning warnStmt = sqlStmt.getWarnings();
			while (warnStmt != null) {
				sbInfo.append(warnStmt + "\n");
				warnStmt = warnStmt.getNextWarning();
				monitor.worked(monitorWorked);
			}
			analyzeOK = true;
			
		} catch (SQLException e) {
			sbInfo.append(e.getMessage());
			e.printStackTrace();
		} finally {
			// Clean up and return to the main run method.
			if (sqlStmt != null) {
				try {
					sqlStmt.close();
				} catch (SQLException e) {
					e.printStackTrace();
					sbInfo.append(e.getMessage() + "\n");
				}
			}
			Application.dbConn.closeConnection();
			sbInfo.append("...End analyze operation.\n");
			if (!analyzeOK) {
				sbInfo.append("\n*** Database analyze operation failed.\n");
			}
		}
		return analyzeOK;
	}
	
	/*
	 * Drops the database in preparation for a restore.
	 */
	private boolean dbDropper(final IProgressMonitor monitor) {
		// Get the parameters needed to perform this operation.
		IPreferenceStore store = KomoPlugin.getDefault().getPreferenceStore();
		String hostString = "-h" + store.getString(PreferenceConstants.DB_URL);
		String portString = "-p" + store.getString(PreferenceConstants.DB_PORT);
		String userString = "-U" + store.getString(PreferenceConstants.DB_USER);
		String dbNameString = store.getString(PreferenceConstants.DB_NAME);
		Process dropProc = null;

		// Initialize a boolean to track the method execution.
		boolean dropOK = false;

		try {
			sbInfo.append("...Begin database delete operation.\nCommand is: " 
					+ dropApp + " " + hostString + " " + portString + " " 
					+ userString + " -W -e " + dbNameString);
			
			// Make sure the database drop utility is present.
			File tempFile = new File(dropApp);
			if (!tempFile.isFile()) {
				sbInfo.append("The database drop utility is not a valid file or could not be found.\n");
				return dropOK;
			}
			
			// Initialize the process.
			ProcessBuilder pb = new ProcessBuilder(dropApp, hostString, portString,
					userString, "-W", "-e", dbNameString);
			pb.redirectErrorStream(true);
			dropProc = pb.start();
			/*
			 * Put a buffered reader on stderr - 
			 * this is where the PostgreSQL log is output. 
			 */
			procReader = new BufferedReader(new InputStreamReader(dropProc
					.getInputStream())); 
			
			// Set up a writer to output the password.
			procWriter = new PrintWriter(dropProc.getOutputStream());
			
			/*
			 * Send password.  Cancel the process if a problem occurs.
			 */
			if (!sendPassword(monitor)) {
				return dropOK;
			}
			
			// Collect information from this process.
			String line = new String();
			while ((line = procReader.readLine()) != null) {
				sbInfo.append(line + "\n");
				monitor.worked((monitorWorked));
			}
			if (dropProc.waitFor() == 0) {
				dropOK = true;
			}
			
		} catch (Exception e) {
			sbInfo.append(e.getMessage() + "\n");
			e.printStackTrace();
			
		} finally {
			// Clean up and return to the main run method.
			if (procReader != null) {
				try {
					procReader.close();
				} catch (IOException e) {
					sbInfo.append(e.getMessage() + "\n");
					e.printStackTrace();
				}
			}
			if (procWriter != null) {
				procWriter.close();
			}
			if (dropProc != null) {
				dropProc.destroy();
			}
			sbInfo.append("...End database delete operation.\n\n");
			if (!dropOK) {
				sbInfo.append("\n*** Database delete operation failed.\n");
			}
		}
		
		return dropOK;
	}

	/*
	 * Handles the password prompt and sends the password to the application.
	 */
	private boolean sendPassword(final IProgressMonitor monitor) {
		// Instantiate variables needed to run the method.
		String tempString = new String();
		int tempInt;
		
		// Initialize a boolean to track the method execution
		boolean passwordOK = false;
		
		try {
			 do {
				tempInt = procReader.read();
				tempString = tempString + (char) tempInt;
				if (tempString.endsWith("Password: ")) {
					procWriter.println(pwString);
					procWriter.flush();
					passwordOK = true;
					sbInfo.append(tempString + " password sent.\n");
					break;
				}
				monitor.worked(monitorWorked);
			 } while (procReader.ready());

		} catch (IOException e) {
			sbInfo.append(tempString + "\n" + e.getMessage() + "\n");
			e.printStackTrace();
		} finally {
			if (!passwordOK) {
				sbInfo.append(
					"\n*** There was a problem with sending the password.\n");
			}
		}
		return passwordOK;
	}

	/*
	 * Show results upon completion of the process,
	 */
	protected final void showResults(final String message) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if (!runOK) {
					String dbMsg = Application.dbConn.getMessage();
					ApplicationActionBarAdvisor.postMessage(dbMsg, true);
				}
				DbInfoDialog dbDialog = new DbInfoDialog(Display.getDefault()
					.getActiveShell(), message);
				dbDialog.create();
				dbDialog.getShell().setText(thisJob + " Results");
				if (thisImage != null && thisImage.length() > 0) {
					dbDialog.getShell().setImage(
							KomoPlugin.getImageDescriptor(thisImage)
									.createImage());
				}
				dbDialog.open();
			}
		});
	}
}
